<>

# HOM

### From GNUstepWiki	{#siteSub}

HOM (short for _High Order Messaging_) is a mechanism for encapsulating control structures and other programming patterns. It is similar to blocks in [Smalltalk][1] and higher order functions in functional languages.

[Objective-C][2] does not have blocks. They're not a part of the language, although [Brad Cox][3]'s [TaskMaster Paper][4] outlines them nicely (and the [Portable Object Compiler][5] implements them).

However, it is possible to implement HOM (using [trampolines][6]), but HOM is not a language feature at this point.

<br />

[[edit][7]]

<>

## What does HOM look like?

Suppose you have an [NSArray][8] containing objects you want to call a message on. Normally you would have to do something like

	 [anArray makeObjectsPerformSelector: @selector(someOtherMessage)];

With HOM you simply write

	 [[anArray do] someOtherMessage];

instead.

Simply put: the _-do_ message tells the array to call _-someOtherMessage_ on each of its objects.

<br />

[[edit][9]]

<>

## HOM related messages

To further explain how HOM works, here's a list of usual methods including a simple example.

We start by assuming there's an array

	 NSArray *array;

	 array = [@"one two three four five six seven eight nine ten" componentsSeparatedByString: @" "];

Now let's see how to use HOM methods on that array and what the results will be:

<br />

* _-select_

Returns members of the array if they respond YES to the argument message -- which necessarily returns a BOOL.

	 NSArray *array2 = [[array select] hasPrefix: @"t"];
	 NSLog(@"%@", array2);

This will give us all numbers starting with a _t_

	 (two, three, ten)

<br />

* _-reject_

The inverse of _-select_; it returns the elements responding NO.

	 NSArray *array3 = [[array2 reject] hasSuffix: @"e"];
	 NSLog(@"%@", array3);

will return all numbers in _array2_ that do **not** end with an _e_

	 (two, ten)

<br />

* _-collect_

Executes the argument message and returns an array of the responses to the argument message.

	 NSArray *array4 = [[array3 collect] stringByAppendingString: @" books"];
	 NSLog(@"%@", array4);

will add _books_ to all objects in _array3_

	 ("two books", "ten books")

<br />

* _-do_

Executes the argument message.

	 NSArray *array5 = [[array collect] mutableCopy];
	 [[array5 do] appendString: @" eggplants"];
	 NSLog(@"%@", array5);

will create a mutable copy of _array_ and append _eggplants_ to each object.

	 (
	   "one eggplants",
	   "two eggplants",
	   "three eggplants",
	   "four eggplants",
	   "five eggplants",
	   "six eggplants",
	   "seven eggplants",
	   "eight eggplants",
	   "nine eggplants",
	   "ten eggplants"
	 )

As you can see, _-do_ is quite similar to _-collect_ but works on the objects inside the collection (here _array5_ -- the mutable copy of _array_) instead of returning a new array (as _-collect_ does).

<br />

* _-each_

Returns each object, similar to _-objectEnumerator_.

	 NSArray *string = [[@"I like to eat " collect]  stringByAppendingString: [array5 each]];

This is a stupid example (because I do not like eggplants) that would give us

	 (
	   "I like to eat one eggplants",
	   "I like to eat two eggplants",
	   "I like to eat three eggplants",
	   "I like to eat four eggplants",
	   "I like to eat five eggplants",
	   "I like to eat six eggplants",
	   "I like to eat seven eggplants",
	   "I like to eat eight eggplants",
	   "I like to eat nine eggplants",
	   "I like to eat ten eggplants"
	 )

<br />

* _-performAfterDelay:_

Performs the argument message after a specified delay.

	 [[someArray performAfterDelay: 5] delayedMessage];

<br />

* _-ignoreExceptions_

Traps and ignores any exception that may occur during execution of the argument message.

	 [[someArray ignoreExceptions] messageThatMayRaiseAnException];

<br />

* _-ifResponds_

Sends the argument message only if the receiver reponds to it. Instead of writing

	 if( [receiver respondsToSelector: @selector(foobar)] ) [receiver foobar]

you simply use

	 [[receiver ifResponds] foobar].

 Retrieved from "<http://wiki.gnustep.org/index.php/HOM>"

[Categories][10]: [Objective-C][11] | [Snippets][12]

##### Views

* [Article][13]
* [Discussion][14]
* [Edit][15]
* [History][16]

##### Personal tools

* [Create an account or log in][17]

[][18]

##### users

* [Applications][19]
* [Guides][20]
* [FAQ][21]
* [Get Help][22]
* [Report Bugs][23]
* [Donations][24]
* [Screenshots][25]

##### developers

* [Guides][26]
* [FAQ][27]
* [Base API (Foundation)][28]
* [GUI API (AppKit)][29]
* [Frameworks][30]

##### Navigation

* [Main Page][31]
* [Community portal][32]
* [Current events][33]
* [Recent changes][34]
* [Random page][35]
* [Help][36]

##### Search

##### Toolbox

* [What links here][37]
* [Related changes][38]
* [Upload file][39]
* [Special pages][40]
* [Printable version][41]

[![MediaWiki][42]][43]

* This page was last modified 18:38, 7 June 2010.
* This page has been accessed 32,127 times.
* Content is available under [GNU General Public License][44].
* [About GNUstepWiki][45]
* [Disclaimers][46]

  [1]: /index.php/Smalltalk "Smalltalk"
  [2]: /index.php/Objective-C "Objective-C"
  [3]: http://virtualschool.edu/cox/ "http://virtualschool.edu/cox/"
  [4]: http://virtualschool.edu/cox/pub/TaskMaster/index.html "http://virtualschool.edu/cox/pub/TaskMaster/index.html"
  [5]: http://users.pandora.be/stes/compiler.html "http://users.pandora.be/stes/compiler.html"
  [6]: /index.php/Trampoline "Trampoline"
  [7]: /index.php?title=HOM&action=edit&section=1 "HOM"
  [8]: /index.php/NSArray "NSArray"
  [9]: /index.php?title=HOM&action=edit&section=2 "HOM"
  [10]: /index.php?title=Special:Categories&article=HOM "Special:Categories"
  [11]: /index.php/Category:Objective-C "Category:Objective-C"
  [12]: /index.php/Category:Snippets "Category:Snippets"
  [13]: /index.php/HOM
  [14]: /index.php?title=Talk:HOM&action=edit
  [15]: /index.php?title=HOM&action=edit
  [16]: /index.php?title=HOM&action=history
  [17]: /index.php?title=Special:Userlogin&returnto=HOM
  [18]: /index.php/Main_Page "Main Page"
  [19]: /index.php/Category:Applications
  [20]: /index.php/User_Guides
  [21]: /index.php/User_FAQ
  [22]: /index.php/Get_Help
  [23]: /index.php/Report_Bugs
  [24]: /index.php/GNUstepWiki:Site_support
  [25]: /index.php/Screenshots
  [26]: /index.php/Developer_Guides
  [27]: /index.php/Developer_FAQ
  [28]: /index.php/Foundation
  [29]: /index.php/AppKit
  [30]: /index.php/Category:Frameworks
  [31]: /index.php/Main_Page
  [32]: /index.php/GNUstepWiki:Community_Portal
  [33]: /index.php/Current_events
  [34]: /index.php/Special:Recentchanges
  [35]: /index.php/Special:Random
  [36]: /index.php/Help:Contents
  [37]: /index.php/Special:Whatlinkshere/HOM
  [38]: /index.php/Special:Recentchangeslinked/HOM
  [39]: /index.php/Special:Upload
  [40]: /index.php/Special:Specialpages
  [41]: /index.php?title=HOM&printable=yes&printable=yes
  [42]: /skins/common/images/poweredby_mediawiki_88x31.png
  [43]: http://www.mediawiki.org/
  [44]: http://www.gnu.org/copyleft/gpl.html "http://www.gnu.org/copyleft/gpl.html"
  [45]: /index.php/GNUstepWiki:About "GNUstepWiki:About"
  [46]: /index.php/GNUstepWiki:General_disclaimer "GNUstepWiki:General disclaimer"